/*
 *  FileSocket.h
 *
 *  Created by David Lapsley on Fri Feb 20 2004.
 *
 */

/* 
 * This software is released under the terms of the MIT License(included below). 
 *  
 * Copyright (c) 2003 MIT Haystack Observatory 
 *  
 * Permission is hereby granted, free of charge, to any person obtaining a   
 * copy of this software and associated documentation files (the "Software"),    
 * to deal in the Software without restriction, including without limitation   
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,   
 * and/or sell copies of the Software, and to permit persons to whom the  
 * Software is furnished to do so, subject to the following conditions: 
 *  
 * The above copyright notice and this permission notice shall be included in    
 * all copies or substantial portions of the Software. 
 *   
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE  
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER   
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 * SOFTWARE. 
 * 
 */

#include <FileSocket.h>
extern "C"{
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/uio.h>
#include <unistd.h>
}

// Constructors/Destructo
FileSocket::FileSocket(): Socket()
{
	 // No socket created.
}

FileSocket::FileSocket(const int& mtu):
  Socket(mtu)
{
  // NO socket created.
}

FileSocket::FileSocket(const int& fd, const int& mtu):
  Socket(fd, mtu)
{
  // NO socket created.
}

FileSocket::~FileSocket()
{    
  // This is an important feature of the TCPSocket class. Especially
    // for copying of TCPSockets. TCPSockets do *not* close their
  // sockets when they are destroyed.
  syslog(LOG_ERR, "FileSocket::~FileSocket(): _bytes_sent==%lld", _bytes_sent);
  syslog(LOG_ERR, "FileSocket::~FileSocket(): _bytes_rcvd==%lld", _bytes_rcvd);
  syslog(LOG_ERR, "FileSocket::~FileSocket(): _send_calls==%lld", _send_calls);
  syslog(LOG_ERR, "FileSocket::~FileSocket(): _recv_calls==%lld", _recv_calls);

  _age.stop();
  double duration=_age.elapsed();
  double rcv_rate=_bytes_rcvd/duration*8.0/1000000;
  double send_rate=_bytes_sent/duration*8.0/1000000;
  syslog(LOG_ERR, "FileSocket::~FileSocket(): duration==%f sec", duration);
  syslog(LOG_ERR, "FileSocket::~FileSocket(): rcv_rate==%f Mbps", rcv_rate);
  syslog(LOG_ERR, "FileSocket::~FileSocket(): send_rate==%f Mbps", send_rate);

  if( Logsystem::Test_Logsys( Logsystem::INFO, 5 )){
    Logsystem::info_logger()->Tstamp()
      << "FileSocket::~FileSocket(): _bytes_sent=" << _bytes_sent
      << "\n\tFileSocket::~FileSocket(): _bytes_rcvd=" <<  _bytes_rcvd
      << "\n\tFileSocket::~FileSocket(): _send_calls=" << _send_calls 
      << "\n\tFileSocket::~FileSocket(): _recv_calls=" << _recv_calls << "\n";
    Logsystem::info_logger()->Tstamp()
      << "FileSocket::~FileSocket(): duration=" <<  duration << " sec.\n"
      << "\tFileSocket::~FileSocket(): rcv_rate=" << rcv_rate << " Mbps\n"
      << "\tFileSocket::~FileSocket(): send_rate=" << send_rate << "Mbps." << "\n";
  }

  if (_sockd!=-1)
    ::close(_sockd);
}

// Member access.
int FileSocket::recv(u_int8_t* b, const int& n)
{
  int ret = ::read(_sockd, b, n);
  _bytes_rcvd += ret;
  return(ret);
}

int FileSocket::send(u_int8_t* const b, const int& n)
{
  int ret = ::write(_sockd, b, n);  
  _bytes_sent += ret;
  return(ret);
}

// Open for Reading only 
int FileSocket::bind(const string& addr, const int& port)
{
  _sockd=open(addr.c_str(), O_RDONLY, S_IRUSR | S_IWUSR);
  if (_sockd==-1) {
    fprintf(stderr,"FileSoket::bind(): Error Cannot open %s\n", addr.c_str());
    syslog(LOG_ERR, "FileSocket::bind():  open error: %s",
   	   strerror(errno));
    return(-1);
  }
  return(0);
}

int FileSocket::listen()
{
  return(0);
}

int FileSocket::accept()
{
  return(_sockd);
}

// Open for output (writing) 
int FileSocket::connect(const string& name, const int& port)
{
  if(name.empty()){
    cerr << "FileSocket::connect() failed: fname is empty\n";
    return -1;
  }
  
  _sockd=open(name.c_str(), O_RDWR | O_TRUNC | O_CREAT | O_APPEND, 
	      S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH);
  if (_sockd>0){
    //    cerr << "Info: FileSocket::connect() opened " << name << endl;
    return(0);
  }
  
  syslog(LOG_ERR, "FileSocket::connect(): open error: %s", strerror(errno));
  return(-1);
}

// FileSocket connection termination...
int FileSocket::close()
{
  if (_sockd==-1) {
    syslog(LOG_ERR, "FileSocket::close() attempt to close unopened descriptor.\n");
    return(-1);
  }
  int ret=::close(_sockd);
  if (ret==0)    _sockd=-1;

  return(ret);
}

// FileSocket connection termination...
int FileSocket::shutdown(const int& howto)
{
  if (_sockd==-1) {
    syslog(LOG_ERR, "FileSocket::shutdown() attempt to shutdown unopened descriptor.\n");
    return(-1);
  }
  int ret=::close(_sockd);
  if (ret==0)   _sockd=-1;

  return(ret);
}
